@@ -746,19 +746,20 @@ namespace xt
     using slice_variant = xtl::variant<
         T,
 
-        xt::xrange_adaptor<xt::placeholders::xtuph, T, T>,
-        xt::xrange_adaptor<T, xt::placeholders::xtuph, T>,
-        xt::xrange_adaptor<T, T, xt::placeholders::xtuph>,
+        xrange_adaptor<placeholders::xtuph, T, T>,
+        xrange_adaptor<T, placeholders::xtuph, T>,
+        xrange_adaptor<T, T, placeholders::xtuph>,
 
-        xt::xrange_adaptor<T, xt::placeholders::xtuph, xt::placeholders::xtuph>,
-        xt::xrange_adaptor<xt::placeholders::xtuph, T, xt::placeholders::xtuph>,
-        xt::xrange_adaptor<xt::placeholders::xtuph, xt::placeholders::xtuph, T>,
+        xrange_adaptor<T, placeholders::xtuph, placeholders::xtuph>,
+        xrange_adaptor<placeholders::xtuph, T, placeholders::xtuph>,
+        xrange_adaptor<placeholders::xtuph, placeholders::xtuph, T>,
 
-        xt::xrange_adaptor<T, T, T>,
-        xt::xrange_adaptor<xt::placeholders::xtuph, xt::placeholders::xtuph, xt::placeholders::xtuph>,
+        xrange_adaptor<T, T, T>,
+        xrange_adaptor<placeholders::xtuph, placeholders::xtuph, placeholders::xtuph>,
 
-        xt::xall_tag,
-        xt::xnewaxis_tag
+        xall_tag,
+        xellipsis_tag,
+        xnewaxis_tag
     >;
 
     /**
@@ -825,9 +826,9 @@ namespace xt
      * auto v = xt::dynamic_view(a, sv);
      * // ==> {{1, 3}}
      * \endcode
-     * 
+     *
      * You can also achieve the same with the following short-hand syntax:
-     * 
+     *
      * \code{.cpp}
      * xt::xarray<double> a = {{1, 2, 3}, {4, 5, 6}};
      * auto v = xt::dynamic_view(a, {xt::range(0, 1), xt::range(0, 3, 2)});
@@ -838,18 +839,45 @@ namespace xt
     inline auto dynamic_view(E&& e, const slice_vector& slices)
     {
         // Compute dimension
-        std::size_t dimension = e.dimension();
-
+        std::size_t dimension = e.dimension(), n_newaxis = 0, n_add_all;
+        ptrdiff_t dimension_check = static_cast<ptrdiff_t>(e.dimension());
+        bool has_ellipsis = false;
         for (const auto& el : slices)
         {
             if (xtl::get_if<xt::xnewaxis_tag>(&el) != nullptr)
             {
                 dimension++;
+                n_newaxis++;
             }
             else if (xtl::get_if<int>(&el) != nullptr)
             {
                 dimension--;
+                dimension_check--;
             }
+            else if (xtl::get_if<xt::xellipsis_tag>(&el) != nullptr)
+            {
+                if (has_ellipsis == true)
+                {
+                    throw std::runtime_error("Ellipsis can only appear once.");
+                }
+                has_ellipsis = true;
+            }
+            else
+            {
+                dimension_check--;
+            }
+        }
+
+        if (dimension_check < 0)
+        {
+            throw std::runtime_error("Too many slices for view.");
+        }
+
+        if (has_ellipsis)
+        {
+            // replace ellipsis with N * xt::all
+            // remove -1 because of the ellipsis slize itself
+            n_add_all = e.dimension() - (slices.size() - 1 - n_newaxis);
         }
 
         // Compute strided view
@@ -862,46 +890,60 @@ namespace xt
         auto old_shape = e.shape();
         auto&& old_strides = detail::get_strides(e);
 
-        std::size_t i = 0, idx = 0, newaxis_skip = 0;
+#define MS(v) static_cast<ptrdiff_t>(v)
+#define MU(v) static_cast<std::size_t>(v)
+
+        ptrdiff_t i = 0, axis_skip = 0;
+        std::size_t idx = 0;
 
         auto slice_getter = detail::slice_getter_impl<E>(e);
 
-        for (; i < slices.size(); ++i)
+        for (; i < MS(slices.size()); ++i)
         {
-            auto ptr = xtl::get_if<int>(&slices[i]);
+            auto ptr = xtl::get_if<int>(&slices[MU(i)]);
             if (ptr != nullptr)
             {
                 std::size_t slice0 = static_cast<std::size_t>(*ptr);
-                offset += slice0 * old_strides[i - newaxis_skip];
+                offset += slice0 * old_strides[MU(i - axis_skip)];
             }
-            else if (xtl::get_if<xt::xnewaxis_tag>(&slices[i]) != nullptr)
+            else if (xtl::get_if<xt::xnewaxis_tag>(&slices[MU(i)]) != nullptr)
             {
                 new_shape[idx] = 1;
                 new_strides[idx] = 0;
-                ++newaxis_skip;
+                ++axis_skip;
                 ++idx;
             }
-            else if (xtl::get_if<xt::xall_tag>(&slices[i]) != nullptr)
+            else if (xtl::get_if<xt::xellipsis_tag>(&slices[MU(i)]) != nullptr)
             {
-                new_shape[idx] = old_shape[i - newaxis_skip];
-                new_strides[idx] = old_strides[i - newaxis_skip];
+                for (std::size_t j = 0; j < n_add_all; ++j)
+                {
+                    new_shape[idx] = old_shape[MU(i - axis_skip)];
+                    new_strides[idx] = old_strides[MU(i - axis_skip)];
+                    ++idx, --axis_skip;
+                }
+                ++axis_skip;  // because i++
+            }
+            else if (xtl::get_if<xt::xall_tag>(&slices[MU(i)]) != nullptr)
+            {
+                new_shape[idx] = old_shape[MU(i - axis_skip)];
+                new_strides[idx] = old_strides[MU(i - axis_skip)];
                 ++idx;
             }
             else
             {
-                slice_getter.idx = i - newaxis_skip;
-                std::array<int, 3> info = xtl::visit(slice_getter, slices[i]);
-                offset += std::size_t(info[0]) * old_strides[i - newaxis_skip];
+                slice_getter.idx = MU(i - axis_skip);
+                std::array<int, 3> info = xtl::visit(slice_getter, slices[MU(i)]);
+                offset += std::size_t(info[0]) * old_strides[MU(i - axis_skip)];
                 new_shape[idx] = std::size_t(info[1]);
-                new_strides[idx] = std::size_t(info[2]) * old_strides[i - newaxis_skip];
+                new_strides[idx] = std::size_t(info[2]) * old_strides[MU(i - axis_skip)];
                 ++idx;
             }
         }
 
-        for (; (i - newaxis_skip) < old_shape.size(); ++i)
+        for (; MU(i - axis_skip) < old_shape.size(); ++i)
         {
-            new_shape[idx] = old_shape[i - newaxis_skip];
-            new_strides[idx] = old_strides[i - newaxis_skip];
+            new_shape[idx] = old_shape[MU(i - axis_skip)];
+            new_strides[idx] = old_strides[MU(i - axis_skip)];
             ++idx;
         }
 
@@ -910,6 +952,9 @@ namespace xt
         using view_type = xstrided_view<xclosure_t<E>, shape_type, decltype(data)>;
         // TODO change layout type?
         return view_type(std::forward<E>(e), std::forward<decltype(data)>(data), std::move(new_shape), std::move(new_strides), offset, layout_type::dynamic);
+
+#undef MU
+#undef MS
     }
 
     /****************************
@@ -1141,6 +1186,260 @@ namespace xt
     {
         return ravel<std::decay_t<E>::static_layout>(std::forward<E>(e));
     }
+
+    /**
+     * Trim zeros at beginning, end or both of 1D sequence.
+     *
+     * @param e input xexpression
+     * @param direction string of either 'f' for trim from beginning, 'b' for trim from end
+     *                  or 'fb' (default) for both.
+     * @return returns a view without zeros at the beginning and end
+     */
+    template <class E>
+    inline auto trim_zeros(E&& e, const std::string& direction = "fb")
+    {
+        XTENSOR_ASSERT_MSG(e.dimension() == 1, "Dimension for trim_zeros has to be 1.");
+
+        ptrdiff_t begin = 0, end = static_cast<ptrdiff_t>(e.size());
+
+        auto find_fun = [](const auto& i) {
+            return i != 0;
+        };
+
+        if (direction.find("f") != std::string::npos)
+        {
+            begin = std::find_if(e.begin(), e.end(), find_fun) - e.begin();
+        }
+
+        if (direction.find("b") != std::string::npos && begin != end)
+        {
+            end -= std::find_if(e.rbegin(), e.rend(), find_fun) - e.rbegin();
+        }
+
+        return dynamic_view(std::forward<E>(e), { range(static_cast<int>(begin), static_cast<int>(end)) });
+    }
+
+    /**
+     * Returns a squeeze view of the given expression. No copy is made.
+     * Squeezing an expression removes dimensions of extent 1.
+     *
+     * @param e the input expression
+     * @tparam E the type of the expression
+     */
+    template <class E>
+    inline auto squeeze(E&& e)
+    {
+        xt::dynamic_shape<std::size_t> new_shape, new_strides;
+        std::copy_if(e.shape().begin(), e.shape().end(), std::back_inserter(new_shape),
+                               [](std::size_t i){ return i != 1; });
+        auto&& old_strides = detail::get_strides(e);
+        std::copy_if(old_strides.begin(), old_strides.end(), std::back_inserter(new_strides),
+                               [](std::size_t i){ return i != 0; });
+        decltype(auto) data = detail::get_data(e);
+
+        using view_type = xstrided_view<xclosure_t<E>, xt::dynamic_shape<std::size_t>, decltype(data)>;
+        return view_type(std::forward<E>(e), std::forward<decltype(data)>(data), std::move(new_shape), std::move(new_strides), 0, e.layout());
+    }
+
+    namespace detail
+    {
+        template <class E, class S>
+        inline auto squeeze_impl(E&& e, S&& axis, check_policy::none)
+        {
+            std::size_t new_dim = e.dimension() - axis.size();
+            xt::dynamic_shape<std::size_t> new_shape(new_dim), new_strides(new_dim);
+
+            decltype(auto) old_strides = detail::get_strides(e);
+            decltype(auto) data = detail::get_data(e);
+
+            for (std::size_t i = 0, ix = 0; i < e.dimension(); ++i)
+            {
+                if (axis.end() == std::find(axis.begin(), axis.end(), i))
+                {
+                    new_shape[ix] = e.shape()[i];
+                    new_strides[ix++] = old_strides[i];
+                }
+            }
+
+            using view_type = xstrided_view<xclosure_t<E>, xt::dynamic_shape<std::size_t>, decltype(data)>;
+            return view_type(std::forward<E>(e), std::forward<decltype(data)>(data), std::move(new_shape), std::move(new_strides), 0, e.layout());
+        }
+
+        template <class E, class S>
+        inline auto squeeze_impl(E&& e, S&& axis, check_policy::full)
+        {
+            for (auto ix : axis)
+            {
+                if (static_cast<std::size_t>(ix) > e.dimension())
+                {
+                    throw std::runtime_error("Axis argument to squeeze > dimension of expression");
+                }
+                if (e.shape()[static_cast<std::size_t>(ix)] != 1)
+                {
+                    throw std::runtime_error("Trying to squeeze axis != 1");
+                }
+            }
+            return squeeze_impl(std::forward<E>(e), std::forward<S>(axis), check_policy::none());
+        }
+    }
+
+    /**
+     * @brief Remove single-dimensional entries from the shape of an xexpression
+     *
+     * @param e input xexpression
+     * @param axis integer or container of integers, select a subset of single-dimensional
+     *        entries of the shape.
+     * @param check_policy select check_policy. With check_policy::full(), selecting an axis
+     *        which is greater than one will throw a runtime_error.
+     */
+    template <class E, class S, class Tag = check_policy::none, std::enable_if_t<!std::is_integral<S>::value, int> = 0>
+    inline auto squeeze(E&& e, S&& axis, Tag check_policy = Tag())
+    {
+        return detail::squeeze_impl(std::forward<E>(e), std::forward<S>(axis), check_policy);
+    }
+
+    /// @cond DOXYGEN_INCLUDE_SFINAE
+#ifdef X_OLD_CLANG
+    template <class E, class I, class Tag = check_policy::none>
+    inline auto squeeze(E&& e, std::initializer_list<I> axis, Tag check_policy = Tag())
+    {
+        dynamic_shape<I> ax(axis);
+        return detail::squeeze_impl(std::forward<E>(e), std::move(ax), check_policy);
+    }
+#else
+    template <class E, class I, std::size_t N, class Tag = check_policy::none>
+    inline auto squeeze(E&& e, const I(&axis)[N], Tag check_policy = Tag())
+    {
+        using arr_t = std::array<I, N>;
+        return detail::squeeze_impl(std::forward<E>(e), xtl::forward_sequence<arr_t>(axis), check_policy);
+    }
+#endif
+
+    template <class E, class Tag = check_policy::none>
+    inline auto squeeze(E&& e, std::size_t axis, Tag check_policy = Tag())
+    {
+        return squeeze(std::forward<E>(e), std::array<std::size_t, 1>({ axis }), check_policy);
+    }
+    /// @endcond
+
+    /**
+     * @brief Expand the shape of an xexpression.
+     *
+     * Insert a new axis that will appear at the axis position in the expanded array shape.
+     * This will return a ``dynamic_view`` with a ``xt::newaxis()`` at the indicated axis.
+     *
+     * @param e input xexpression
+     * @param axis axis to expand
+     * @return returns a ``dynamic_view`` with expanded dimension
+     */
+    template <class E>
+    auto expand_dims(E&& e, std::size_t axis)
+    {
+        slice_vector sv(e.dimension() + 1, xt::all());
+        sv[axis] = xt::newaxis();
+        return dynamic_view(std::forward<E>(e), std::move(sv));
+    }
+
+    /**
+     * Expand dimensions of xexpression to at least `N`
+     *
+     * This adds ``newaxis()`` slices to a ``dynamic_view`` until
+     * the dimension of the view reaches at least `N`.
+     * Note: dimensions are added equally at the beginning and the end.
+     * For example, a 1-D array of shape (N,) becomes a view of shape (1, N, 1).
+     *
+     * @param e input xexpression
+     * @tparam N the number of requested dimensions
+     * @return ``dynamic_view`` with expanded dimensions
+     */
+    template <std::size_t N, class E>
+    auto atleast_Nd(E&& e)
+    {
+        slice_vector sv(std::max(e.dimension(), N), xt::all());
+        if (e.dimension() < N)
+        {
+            std::size_t i = 0;
+            for (; i < std::round(double((N - e.dimension()) / double(N))); ++i)
+            {
+                sv[i] = xt::newaxis();
+            }
+            i += e.dimension();
+            for (; i < N; ++i)
+            {
+                sv[i] = xt::newaxis();
+            }
+        }
+        return dynamic_view(std::forward<E>(e), std::move(sv));
+    }
+
+    /**
+     * Expand to at least 1D
+     * @sa atleast_Nd
+     */
+    template <class E>
+    auto atleast_1d(E&& e)
+    {
+        return atleast_Nd<1>(std::forward<E>(e));
+    }
+
+    /**
+     * Expand to at least 2D
+     * @sa atleast_Nd
+     */
+    template <class E>
+    auto atleast_2d(E&& e)
+    {
+        return atleast_Nd<2>(std::forward<E>(e));
+    }
+
+    /**
+     * Expand to at least 3D
+     * @sa atleast_Nd
+     */
+    template <class E>
+    auto atleast_3d(E&& e)
+    {
+        return atleast_Nd<3>(std::forward<E>(e));
+    }
+
+    /**
+     * @brief Split xexpression along axis into subexpressions
+     *
+     * This splits an xexpression along the axis in `n` equal parts and
+     * returns a vector of ``dynamic_view``.
+     * Calling split with axis > dimension of e or a `n` that does not result in
+     * an equal division of the xexpression will throw a runtime_error.
+     *
+     * @param e input xexpression
+     * @param n number of elements to return
+     * @param axis axis along which to split the expression
+     */
+    template <class E>
+    auto split(E& e, std::size_t n, std::size_t axis = 0)
+    {
+        if (axis >= e.dimension())
+        {
+            throw std::runtime_error("Split along axis > dimension.");
+        }
+
+        std::size_t ax_sz = e.shape()[axis];
+        slice_vector sv(e.dimension(), xt::all());
+        std::size_t step = ax_sz / n;
+        std::size_t rest = ax_sz % n;
+
+        if (rest)
+        {
+            throw std::runtime_error("Split does not result in equal division.");
+        }
+
+        std::vector<decltype(dynamic_view(e, sv))> result;
+        for (std::size_t i = 0; i < n; ++i)
+        {
+            sv[axis] = range(static_cast<int>(i * step), static_cast<int>((i + 1) * step));
+            result.emplace_back(dynamic_view(e, sv));
+        }
+        return result;
+    }
 }
 
 #endif
